"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-duplicate-imports
const office_1 = require("../../api/office");
const style_1 = require("../../api/style");
const style_2 = require("../../api/style");
const style_3 = require("../../api/style");
const OdfAttributeName_1 = require("../OdfAttributeName");
const OdfElementName_1 = require("../OdfElementName");
/**
 * Transforms a {@link StyleManager} object into ODF conform XML
 *
 * NOTE: The properties are set in the order of their appearance in the Realx NG schema.
 *
 * @since 0.9.0
 */
class StylesWriter {
    /**
     * Transforms the given {@link FontFaceDeclarations} into Open Document Format.
     *
     * @param {IStyles} styles The styles to serialize
     * @param {Document} document The XML document
     * @param {Element} parent The parent node in the DOM
     * @since 0.7.0
     */
    write(styles, document, root) {
        const allStyles = styles.getAll();
        if (allStyles.length === 0) {
            return;
        }
        const tagName = styles instanceof office_1.CommonStyles ? OdfElementName_1.OdfElementName.OfficeStyles : OdfElementName_1.OdfElementName.OfficeAutomaticStyles;
        const stylesElement = document.createElement(tagName);
        root.appendChild(stylesElement);
        allStyles.forEach((style) => this.visitStyle(style, styles, document, stylesElement));
    }
    visitStyle(style, styles, document, parent) {
        const styleElement = document.createElement(OdfElementName_1.OdfElementName.StyleStyle);
        parent.appendChild(styleElement);
        const styleName = style.getName();
        if (styleName === style_2.Style.UNNAMED) {
            styleElement.setAttribute('style:name', styles.getName(style));
        }
        else {
            styleElement.setAttribute('style:name', style.getName());
            styleElement.setAttribute('style:display-name', style.getDisplayName());
        }
        styleElement.setAttribute('style:family', style.getFamily());
        const clazz = style.getClass();
        if (clazz !== undefined) {
            styleElement.setAttribute('style:class', clazz);
        }
        switch (style.getFamily()) {
            case style_2.StyleFamily.Paragraph:
                this.visitParagraphProperties(style, document, styleElement);
                this.visitTextProperties(style, document, styleElement);
                break;
            default:
                break;
        }
        return styleElement;
    }
    visitParagraphProperties(style, document, parent) {
        const paragraphPropertiesElement = document.createElement(OdfElementName_1.OdfElementName.StyleParagraphProperties);
        parent.appendChild(paragraphPropertiesElement);
        const lineHeight = style.getLineHeight();
        switch (typeof lineHeight) {
            case 'number':
                paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatLineHeight, lineHeight + 'mm');
                break;
            case 'string':
                paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatLineHeight, lineHeight);
                break;
            default:
                break;
        }
        const lineHeightAtLeast = style.getLineHeightAtLeast();
        if (lineHeightAtLeast !== undefined) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleLineHeightAtLeast, lineHeightAtLeast + 'mm');
        }
        const lineSpacing = style.getLineSpacing();
        if (lineSpacing !== undefined) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleLineSpacing, lineSpacing + 'mm');
        }
        const horizontalAlignment = style.getHorizontalAlignment();
        if (horizontalAlignment !== style_1.HorizontalAlignment.Default) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatTextAlign, horizontalAlignment);
        }
        const horizontalAlignmentLastLine = style.getHorizontalAlignmentLastLine();
        if (horizontalAlignment === style_1.HorizontalAlignment.Justify
            && horizontalAlignmentLastLine !== style_1.HorizontalAlignmentLastLine.Default) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatTextAlignLast, horizontalAlignmentLastLine);
        }
        if (style.getKeepTogether() === true) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatKeepTogether, 'always');
        }
        const widows = style.getWidows();
        if (widows !== undefined) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatWidows, widows.toString(10));
        }
        const orphans = style.getOrphans();
        if (orphans !== undefined) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatOrphans, orphans.toString(10));
        }
        const marginLeft = style.getMarginLeft();
        if (marginLeft !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatMarginLeft, marginLeft.toString(10) + 'mm');
        }
        const marginRight = style.getMarginRight();
        if (marginRight !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatMarginRight, marginRight.toString(10) + 'mm');
        }
        const textIndent = style.getTextIndent();
        if (textIndent !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatTextIndent, textIndent.toString(10) + 'mm');
        }
        const marginTop = style.getMarginTop();
        if (marginTop !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatMarginTop, marginTop.toString(10) + 'mm');
        }
        const marginBottom = style.getMarginBottom();
        if (marginBottom !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatMarginBottom, marginBottom.toString(10) + 'mm');
        }
        switch (style.getPageBreak()) {
            case style_1.PageBreak.Before:
                paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBreakBefore, 'page');
                break;
            case style_1.PageBreak.After:
                paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBreakAfter, 'page');
                break;
            default:
                break;
        }
        const backgroundColor = style.getBackgroundColor();
        if (backgroundColor !== undefined) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBackgroundColor, backgroundColor.toHex());
        }
        const borderTop = style.getBorderTop();
        if (borderTop !== undefined && borderTop.width > 0 && borderTop.style !== style_1.BorderStyle.None) {
            const border = `${borderTop.width}mm ${borderTop.style} ${borderTop.color.toHex()}`;
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBorderTop, border);
        }
        const borderBottom = style.getBorderBottom();
        if (borderBottom !== undefined && borderBottom.width > 0 && borderBottom.style !== style_1.BorderStyle.None) {
            const border = `${borderBottom.width}mm ${borderBottom.style} ${borderBottom.color.toHex()}`;
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBorderBottom, border);
        }
        const borderLeft = style.getBorderLeft();
        if (borderLeft !== undefined && borderLeft.width > 0 && borderLeft.style !== style_1.BorderStyle.None) {
            const border = `${borderLeft.width}mm ${borderLeft.style} ${borderLeft.color.toHex()}`;
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBorderLeft, border);
        }
        const borderRight = style.getBorderRight();
        if (borderRight !== undefined && borderRight.width > 0 && borderRight.style !== style_1.BorderStyle.None) {
            const border = `${borderRight.width}mm ${borderRight.style} ${borderRight.color.toHex()}`;
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBorderRight, border);
        }
        const paddingLeft = style.getPaddingLeft();
        if (paddingLeft !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatPaddingLeft, paddingLeft.toString(10) + 'mm');
        }
        const paddingRight = style.getPaddingRight();
        if (paddingRight !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatPaddingRight, paddingRight.toString(10) + 'mm');
        }
        const paddingTop = style.getPaddingTop();
        if (paddingTop !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatPaddingTop, paddingTop.toString(10) + 'mm');
        }
        const paddingBottom = style.getPaddingBottom();
        if (paddingBottom !== 0) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatPaddingBottom, paddingBottom.toString(10) + 'mm');
        }
        if (style.getKeepWithNext() === true) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatKeepWithNext, 'always');
        }
        if (style.getVerticalAlignment() !== style_3.VerticalAlignment.Default) {
            paragraphPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleVerticalAlign, style.getVerticalAlignment());
        }
        const tabStops = style.getTabStops();
        if (tabStops.length === 0) {
            return paragraphPropertiesElement;
        }
        const tabStopsElement = document.createElement(OdfElementName_1.OdfElementName.StyleTabStops);
        paragraphPropertiesElement.appendChild(tabStopsElement);
        tabStops.forEach((tabStop) => {
            const tabStopElement = document.createElement(OdfElementName_1.OdfElementName.StyleTabStop);
            tabStopsElement.appendChild(tabStopElement);
            tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StylePosition, tabStop.getPosition() + 'mm');
            const type = tabStop.getType();
            const char = tabStop.getChar();
            if (type === style_2.TabStopType.Char && char !== undefined) {
                tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleType, type);
                tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleChar, char);
            }
            else if (type !== style_2.TabStopType.Left) {
                tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleType, type);
            }
            const leaderStyle = tabStop.getLeaderStyle();
            if (leaderStyle !== style_2.TabStopLeaderStyle.None) {
                tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleLeaderStyle, leaderStyle);
            }
            const leaderColor = tabStop.getLeaderColor();
            if (leaderColor !== undefined) {
                tabStopElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleLeaderColor, leaderColor.toHex());
            }
        });
        return paragraphPropertiesElement;
    }
    visitTextProperties(style, document, parent) {
        const textPropertiesElement = document.createElement(OdfElementName_1.OdfElementName.StyleTextProperties);
        parent.appendChild(textPropertiesElement);
        const fontVariant = style.getFontVariant();
        if (fontVariant !== style_1.FontVariant.Normal) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatFontVariant, fontVariant);
        }
        const textTransformation = style.getTextTransformation();
        if (textTransformation !== style_3.TextTransformation.None) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatTextTransform, textTransformation);
        }
        const color = style.getColor();
        if (color !== undefined) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatColor, color.toHex());
        }
        const fontName = style.getFontName();
        if (fontName !== undefined) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.StyleFontName, fontName);
        }
        const fontSize = style.getFontSize();
        if (fontSize !== 12) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatFontSize, fontSize + 'pt');
        }
        const typeface = style.getTypeface();
        if (typeface === style_3.Typeface.Italic || typeface === style_3.Typeface.BoldItalic) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatFontStyle, 'italic');
        }
        if (typeface === style_3.Typeface.Oblique || typeface === style_3.Typeface.BoldOblique) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatFontStyle, 'oblique');
        }
        if (typeface === style_3.Typeface.Bold
            || typeface === style_3.Typeface.BoldItalic
            || typeface === style_3.Typeface.BoldOblique) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatFontWeight, 'bold');
        }
        const backgroundColor = style.getBackgroundColor();
        if (backgroundColor !== undefined && !(style instanceof style_2.ParagraphStyle)) {
            textPropertiesElement.setAttribute(OdfAttributeName_1.OdfAttributeName.FormatBackgroundColor, backgroundColor.toHex());
        }
        return textPropertiesElement;
    }
}
exports.StylesWriter = StylesWriter;
